"""Autonomous Futures Protocol Python SDK."""

from . import bindings
from .afp import AFP
from .auth import Authenticator, KeyfileAuthenticator, PrivateKeyAuthenticator
from .exceptions import AFPException

__all__ = (
    "bindings",
    "AFP",
    "AFPException",
    "Authenticator",
    "KeyfileAuthenticator",
    "PrivateKeyAuthenticator",
)



class AFPException(Exception):
    pass


class ConfigurationError(AFPException):
    pass


class ClearingSystemError(AFPException):
    pass


class ExchangeError(AFPException):
    pass


class AuthenticationError(ExchangeError):
    pass


class AuthorizationError(ExchangeError):
    pass


class NotFoundError(ExchangeError):
    pass


class ValidationError(ExchangeError):
    pass



from unittest.mock import Mock

from requests import Response
from requests.adapters import HTTPAdapter

from afp.exchange import ExchangeClient


def test_send_request(monkeypatch):
    fake_response = Response()
    fake_response.status_code = 200
    mock_send = Mock(return_value=fake_response)

    monkeypatch.setattr(HTTPAdapter, "send", mock_send)

    client = ExchangeClient("http://bar/")
    client._send_request("POST", "/orders", api_version=2, data="hello")

    mock_send.assert_called_once()
    request = mock_send.call_args_list[0].args[0]
    assert request.method == "POST"
    assert request.url == "http://bar/v2/orders"
    assert request.body == "hello"



import warnings
from datetime import datetime
from decimal import Decimal
from typing import Any, cast

from eth_typing.evm import ChecksumAddress
from hexbytes import HexBytes
from web3 import Web3

from .. import constants, hashing, validators
from ..bindings import (
    ClearingDiamond,
    OracleSpecification,
    Product as OnChainProduct,
    ProductMetadata,
    ProductRegistry,
)
from ..bindings.erc20 import ERC20
from ..bindings.facade import CLEARING_DIAMOND_ABI
from ..bindings.product_registry import ABI as PRODUCT_REGISTRY_ABI
from ..decorators import convert_web3_error
from ..exceptions import NotFoundError
from ..schemas import ProductSpecification, Transaction
from .base import ClearingSystemAPI


class Product(ClearingSystemAPI):
    """API for managing products."""

    ### Factories ###

    @convert_web3_error()
    def create(
        self,
        *,
        symbol: str,
        description: str,
        fsv_decimals: int,
        fsp_alpha: Decimal,
        fsp_beta: Decimal,
        fsv_calldata: str,
        start_time: datetime,
        earliest_fsp_submission_time: datetime,
        collateral_asset: str,
        tick_size: int,
        unit_value: Decimal,
        initial_margin_requirement: Decimal,
        maintenance_margin_requirement: Decimal,
        auction_bounty: Decimal,
        tradeout_interval: int,
        extended_metadata: str,
        oracle_address: str | None = None,
    ) -> ProductSpecification:
        """Creates a product specification with the given product data.

        The builder account's address is derived from the private key; the price
        quotation symbol is retrieved from the collateral asset.

        Parameters
        ----------
        symbol : str
        description : str
        fsv_decimals: int
        fsp_alpha: Decimal
        fsp_beta: int
        fsv_calldata: str
        start_time : datetime
        earliest_fsp_submission_time : datetime
        collateral_asset : str
        tick_size : int
        unit_value : Decimal
        initial_margin_requirement : Decimal
        maintenance_margin_requirement : Decimal
        auction_bounty : Decimal
        tradeout_interval : int
        extended_metadata : str
        oracle_address: str, optional

        Returns
        -------
        afp.schemas.ProductSpecification
        """
        if oracle_address is None:
            oracle_address = self._config.oracle_provider_address

        if len(self._w3.eth.get_code(Web3.to_checksum_address(collateral_asset))) == 0:
            raise NotFoundError(f"No ERC20 token found at address {collateral_asset}")
        if len(self._w3.eth.get_code(Web3.to_checksum_address(oracle_address))) == 0:
            raise NotFoundError(f"No contract found at oracle address {oracle_address}")

        erc20_contract = ERC20(self._w3, Web3.to_checksum_address(collateral_asset))
        price_quotation = erc20_contract.symbol()

        product_id = Web3.to_hex(
            hashing.generate_product_id(self._authenticator.address, symbol)
        )

        return ProductSpecification(
            id=product_id,
            builder_id=self._authenticator.address,
            symbol=symbol,
            description=description,
            oracle_address=oracle_address,
            fsv_decimals=fsv_decimals,
            fsp_alpha=fsp_alpha,
            fsp_beta=fsp_beta,
            fsv_calldata=fsv_calldata,
            price_quotation=price_quotation,
            collateral_asset=collateral_asset,
            start_time=start_time,
            earliest_fsp_submission_time=earliest_fsp_submission_time,
            tick_size=tick_size,
            unit_value=unit_value,
            initial_margin_requirement=initial_margin_requirement,
            maintenance_margin_requirement=maintenance_margin_requirement,
            auction_bounty=auction_bounty,
            tradeout_interval=tradeout_interval,
            extended_metadata=extended_metadata,
        )

    def create_product(self, **kwargs: Any) -> ProductSpecification:
        """Deprecated alias of `Product.create`."""
        warnings.warn(
            "Product.create_product() is deprecated. Use Product.create() instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.create(**kwargs)

    ### Transactions ###

    @convert_web3_error(PRODUCT_REGISTRY_ABI)
    def register(self, product_specification: ProductSpecification) -> Transaction:
        """Submits a product specification to the clearing system.

        Parameters
        ----------
        product_specification : afp.schemas.ProductSpecification

        Returns
        -------
        afp.schemas.Transaction
            Transaction parameters.
        """
        erc20_contract = ERC20(
            self._w3, cast(ChecksumAddress, product_specification.collateral_asset)
        )
        decimals = erc20_contract.decimals()

        product_registry_contract = ProductRegistry(
            self._w3, self._config.product_registry_address
        )
        return self._transact(
            product_registry_contract.register(
                self._convert_product_specification(product_specification, decimals)
            )
        )

    def register_product(self, product: ProductSpecification) -> Transaction:
        """Deprecated alias of `Product.register`."""
        warnings.warn(
            "Product.register_product() is deprecated. Use Product.register() instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.register(product)

    @convert_web3_error(CLEARING_DIAMOND_ABI)
    def initiate_final_settlement(
        self, product_id: str, accounts: list[str]
    ) -> Transaction:
        """Initiate final settlement (closeout) process for the specified accounts.

        The product must be in Final Settlement state. The accounts must hold non-zero
        positions in the product that offset each other (i.e. the sum of their position
        sizes is 0.)

        Parameters
        ----------
        product_id : str
            The ID of the product.
        accounts : list of str
            List of margin account IDs to initiate settlement for.

        Returns
        -------
        afp.schemas.Transaction
            Transaction parameters.
        """
        product_id = validators.validate_hexstr32(product_id)
        addresses = [validators.validate_address(account) for account in accounts]

        clearing_contract = ClearingDiamond(
            self._w3, self._config.clearing_diamond_address
        )
        return self._transact(
            clearing_contract.initiate_final_settlement(HexBytes(product_id), addresses)
        )

    ### Views ###

    @convert_web3_error(PRODUCT_REGISTRY_ABI)
    def state(self, product_id: str) -> str:
        """Returns the current state of a product.

        Parameters
        ----------
        product_id : str
            The ID of the product.

        Returns
        -------
        str
        """
        product_id = validators.validate_hexstr32(product_id)
        product_registry_contract = ProductRegistry(
            self._w3, self._config.product_registry_address
        )
        state = product_registry_contract.state(HexBytes(product_id))
        return state.name

    def product_state(self, product_id: str) -> str:
        """Deprecated alias of `Product.state`."""
        warnings.warn(
            "Product.product_state() is deprecated. Use Product.state() instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.state(product_id)

    @convert_web3_error(PRODUCT_REGISTRY_ABI)
    def collateral_asset(self, product_id: str) -> str:
        """Returns the collateral asset of a product.

        Parameters
        ----------
        product_id : str
            The ID of the product.

        Returns
        -------
        str
        """
        product_id = validators.validate_hexstr32(product_id)
        product_registry_contract = ProductRegistry(
            self._w3, self._config.product_registry_address
        )
        collateral_asset = product_registry_contract.collateral_asset(
            HexBytes(product_id)
        )
        if Web3.to_int(hexstr=collateral_asset) == 0:
            raise NotFoundError("Product not found in the product registry")
        return collateral_asset

    ### Internal helpers ###

    @staticmethod
    def _convert_product_specification(
        product: ProductSpecification, decimals: int
    ) -> OnChainProduct:
        return OnChainProduct(
            metadata=ProductMetadata(
                builder=cast(ChecksumAddress, product.builder_id),
                symbol=product.symbol,
                description=product.description,
            ),
            oracle_spec=OracleSpecification(
                oracle_address=cast(ChecksumAddress, product.oracle_address),
                fsv_decimals=product.fsv_decimals,
                fsp_alpha=int(product.fsp_alpha * constants.FULL_PRECISION_MULTIPLIER),
                fsp_beta=int(product.fsp_beta * 10**product.fsv_decimals),
                fsv_calldata=HexBytes(product.fsv_calldata),
            ),
            price_quotation=product.price_quotation,
            collateral_asset=cast(ChecksumAddress, product.collateral_asset),
            start_time=int(product.start_time.timestamp()),
            earliest_fsp_submission_time=int(
                product.earliest_fsp_submission_time.timestamp()
            ),
            tick_size=product.tick_size,
            unit_value=int(product.unit_value * 10**decimals),
            initial_margin_requirement=int(
                product.initial_margin_requirement * constants.RATE_MULTIPLIER
            ),
            maintenance_margin_requirement=int(
                product.maintenance_margin_requirement * constants.RATE_MULTIPLIER
            ),
            auction_bounty=int(product.auction_bounty * constants.RATE_MULTIPLIER),
            tradeout_interval=product.tradeout_interval,
            extended_metadata=product.extended_metadata,
        )



import json
import os
from typing import Protocol, cast

from eth_account.account import Account
from eth_account.datastructures import SignedTransaction
from eth_account.messages import encode_defunct
from eth_account.signers.local import LocalAccount
from eth_account.types import TransactionDictType
from eth_typing.evm import ChecksumAddress
from hexbytes import HexBytes
from web3.types import TxParams


class Authenticator(Protocol):
    address: ChecksumAddress

    def sign_message(self, message: bytes) -> HexBytes: ...

    def sign_transaction(self, params: TxParams) -> SignedTransaction: ...


class PrivateKeyAuthenticator(Authenticator):
    """Authenticates with a private key specified in a constructor argument.

    Parameters
    ----------
    private_key: str
        The private key of a blockchain account.
    """

    _account: LocalAccount

    def __init__(self, private_key: str) -> None:
        self._account = Account.from_key(private_key)
        self.address = self._account.address

    def sign_message(self, message: bytes) -> HexBytes:
        eip191_message = encode_defunct(message)
        signed_message = self._account.sign_message(eip191_message)
        return signed_message.signature

    def sign_transaction(self, params: TxParams) -> SignedTransaction:
        return self._account.sign_transaction(cast(TransactionDictType, params))

    def __repr__(self):
        return f"{self.__class__.__name__}(address='{self.address}')"


class KeyfileAuthenticator(PrivateKeyAuthenticator):
    """Authenticates with a private key read from an encrypted keyfile.

    Parameters
    ----------
    key_file : str
        The path to the keyfile.
    password : str
        The password for decrypting the keyfile.
    """

    def __init__(self, key_file: str, password: str) -> None:
        with open(os.path.expanduser(key_file), encoding="utf8") as f:
            key_data = json.load(f)

        private_key = Account.decrypt(key_data, password=password)
        super().__init__(private_key.to_0x_hex())



import os
from types import SimpleNamespace


def _int_or_none(value: str | None) -> int | None:
    return int(value) if value is not None else None


USER_AGENT = "afp-sdk"
DEFAULT_EXCHANGE_API_VERSION = 1

# Constants from clearing/contracts/lib/constants.sol
RATE_MULTIPLIER = 10**4
FEE_RATE_MULTIPLIER = 10**6
FULL_PRECISION_MULTIPLIER = 10**18

defaults = SimpleNamespace(
    # Authentication parameters
    KEYFILE=os.getenv("AFP_KEYFILE", None),
    KEYFILE_PASSWORD=os.getenv("AFP_KEYFILE_PASSWORD", ""),
    PRIVATE_KEY=os.getenv("AFP_PRIVATE_KEY", None),
    # Venue parameters
    EXCHANGE_URL=os.getenv(
        "AFP_EXCHANGE_URL", "https://afp-exchange-stable.up.railway.app"
    ),
    # Blockchain parameters
    RPC_URL=os.getenv("AFP_RPC_URL", None),
    CHAIN_ID=int(os.getenv("AFP_CHAIN_ID", 65000000)),
    GAS_LIMIT=_int_or_none(os.getenv("AFP_GAS_LIMIT", None)),
    MAX_FEE_PER_GAS=_int_or_none(os.getenv("AFP_MAX_FEE_PER_GAS", None)),
    MAX_PRIORITY_FEE_PER_GAS=_int_or_none(
        os.getenv("AFP_MAX_PRIORITY_FEE_PER_GAS", None)
    ),
    TIMEOUT_SECONDS=int(os.getenv("AFP_TIMEOUT_SECONDS", 10)),
    # Clearing System parameters
    CLEARING_DIAMOND_ADDRESS=os.getenv(
        "AFP_CLEARING_DIAMOND_ADDRESS", "0x5B5411F1548254d25360d71FE40cFc1cC983B2A2"
    ),
    MARGIN_ACCOUNT_REGISTRY_ADDRESS=os.getenv(
        "AFP_MARGIN_ACCOUNT_REGISTRY_ADDRESS",
        "0x99f4FA9Cdce7AD227eB84907936a8FeF2095D846",
    ),
    ORACLE_PROVIDER_ADDRESS=os.getenv(
        "AFP_ORACLE_PROVIDER_ADDRESS", "0xF2A2A27da33D30B4BF38D7e186E7B0b1e964e55c"
    ),
    PRODUCT_REGISTRY_ADDRESS=os.getenv(
        "AFP_PRODUCT_REGISTRY_ADDRESS", "0x86B3829471929B115367DA0958f56A6AB844b08e"
    ),
    SYSTEM_VIEWER_ADDRESS=os.getenv(
        "AFP_SYSTEM_VIEWER_ADDRESS", "0xfF2DFcC44a95cce96E03EfC33C65c8Be671Bae5B"
    ),
)



"""OracleProvider contract binding and data structures."""

# This module has been generated using pyabigen v0.2.16

import typing

import eth_typing
import hexbytes
import web3
from web3.contract import contract


class OracleProvider:
    """OracleProvider contract binding.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed OracleProvider contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def Initialized(self) -> contract.ContractEvent:
        """Binding for `event Initialized` on the OracleProvider contract."""
        return self._contract.events.Initialized

    @property
    def OwnershipTransferred(self) -> contract.ContractEvent:
        """Binding for `event OwnershipTransferred` on the OracleProvider contract."""
        return self._contract.events.OwnershipTransferred

    @property
    def PriceSubmitted(self) -> contract.ContractEvent:
        """Binding for `event PriceSubmitted` on the OracleProvider contract."""
        return self._contract.events.PriceSubmitted

    @property
    def Upgraded(self) -> contract.ContractEvent:
        """Binding for `event Upgraded` on the OracleProvider contract."""
        return self._contract.events.Upgraded

    def upgrade_interface_version(
        self,
    ) -> str:
        """Binding for `UPGRADE_INTERFACE_VERSION` on the OracleProvider contract.

        Returns
        -------
        str
        """
        return_value = self._contract.functions.UPGRADE_INTERFACE_VERSION().call()
        return str(return_value)

    def get_admin(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `getAdmin` on the OracleProvider contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.getAdmin().call()
        return eth_typing.ChecksumAddress(return_value)

    def initialize(
        self,
        _product_registry: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `initialize` on the OracleProvider contract.

        Parameters
        ----------
        _product_registry : eth_typing.ChecksumAddress

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.initialize(
            _product_registry,
        )

    def is_admin_active(
        self,
    ) -> bool:
        """Binding for `isAdminActive` on the OracleProvider contract.

        Returns
        -------
        bool
        """
        return_value = self._contract.functions.isAdminActive().call()
        return bool(return_value)

    def owner(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `owner` on the OracleProvider contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.owner().call()
        return eth_typing.ChecksumAddress(return_value)

    def product_registry(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `productRegistry` on the OracleProvider contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.productRegistry().call()
        return eth_typing.ChecksumAddress(return_value)

    def proxiable_uuid(
        self,
    ) -> hexbytes.HexBytes:
        """Binding for `proxiableUUID` on the OracleProvider contract.

        Returns
        -------
        hexbytes.HexBytes
        """
        return_value = self._contract.functions.proxiableUUID().call()
        return hexbytes.HexBytes(return_value)

    def renounce_ownership(
        self,
    ) -> contract.ContractFunction:
        """Binding for `renounceOwnership` on the OracleProvider contract.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.renounceOwnership()

    def resolve(
        self,
        product_id: hexbytes.HexBytes,
        key1: hexbytes.HexBytes,
    ) -> int:
        """Binding for `resolve` on the OracleProvider contract.

        Parameters
        ----------
        product_id : hexbytes.HexBytes
        key1 : hexbytes.HexBytes

        Returns
        -------
        int
        """
        return_value = self._contract.functions.resolve(
            product_id,
            key1,
        ).call()
        return int(return_value)

    def set_active(
        self,
        active: bool,
    ) -> contract.ContractFunction:
        """Binding for `setActive` on the OracleProvider contract.

        Parameters
        ----------
        active : bool

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setActive(
            active,
        )

    def set_admin(
        self,
        admin: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `setAdmin` on the OracleProvider contract.

        Parameters
        ----------
        admin : eth_typing.ChecksumAddress

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setAdmin(
            admin,
        )

    def submit(
        self,
        key: hexbytes.HexBytes,
        fsp: int,
    ) -> contract.ContractFunction:
        """Binding for `submit` on the OracleProvider contract.

        Parameters
        ----------
        key : hexbytes.HexBytes
        fsp : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.submit(
            key,
            fsp,
        )

    def transfer_ownership(
        self,
        new_owner: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `transferOwnership` on the OracleProvider contract.

        Parameters
        ----------
        new_owner : eth_typing.ChecksumAddress

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.transferOwnership(
            new_owner,
        )

    def upgrade_to_and_call(
        self,
        new_implementation: eth_typing.ChecksumAddress,
        data: hexbytes.HexBytes,
    ) -> contract.ContractFunction:
        """Binding for `upgradeToAndCall` on the OracleProvider contract.

        Parameters
        ----------
        new_implementation : eth_typing.ChecksumAddress
        data : hexbytes.HexBytes

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.upgradeToAndCall(
            new_implementation,
            data,
        )


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {"internalType": "address", "name": "target", "type": "address"}
            ],
            "name": "AddressEmptyCode",
            "type": "error",
        },
        {
            "inputs": [
                {"internalType": "bytes4", "name": "selector", "type": "bytes4"},
                {"internalType": "address", "name": "sender", "type": "address"},
            ],
            "name": "AdminControlledNotAuthorized",
            "type": "error",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "implementation", "type": "address"}
            ],
            "name": "ERC1967InvalidImplementation",
            "type": "error",
        },
        {"inputs": [], "name": "ERC1967NonPayable", "type": "error"},
        {
            "inputs": [
                {"internalType": "bytes32", "name": "productId", "type": "bytes32"}
            ],
            "name": "FSPNotFound",
            "type": "error",
        },
        {"inputs": [], "name": "FailedCall", "type": "error"},
        {"inputs": [], "name": "InvalidInitialization", "type": "error"},
        {
            "inputs": [
                {"internalType": "enum ProductState", "name": "state", "type": "uint8"}
            ],
            "name": "InvalidProductState",
            "type": "error",
        },
        {"inputs": [], "name": "NotInitializing", "type": "error"},
        {
            "inputs": [{"internalType": "address", "name": "owner", "type": "address"}],
            "name": "OwnableInvalidOwner",
            "type": "error",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "account", "type": "address"}
            ],
            "name": "OwnableUnauthorizedAccount",
            "type": "error",
        },
        {"inputs": [], "name": "UUPSUnauthorizedCallContext", "type": "error"},
        {
            "inputs": [{"internalType": "bytes32", "name": "slot", "type": "bytes32"}],
            "name": "UUPSUnsupportedProxiableUUID",
            "type": "error",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "uint64",
                    "name": "version",
                    "type": "uint64",
                }
            ],
            "name": "Initialized",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "previousOwner",
                    "type": "address",
                },
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "newOwner",
                    "type": "address",
                },
            ],
            "name": "OwnershipTransferred",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "bytes32",
                    "name": "key",
                    "type": "bytes32",
                },
                {
                    "indexed": False,
                    "internalType": "int256",
                    "name": "fsp",
                    "type": "int256",
                },
                {
                    "indexed": False,
                    "internalType": "address",
                    "name": "submitter",
                    "type": "address",
                },
            ],
            "name": "PriceSubmitted",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "implementation",
                    "type": "address",
                }
            ],
            "name": "Upgraded",
            "type": "event",
        },
        {
            "inputs": [],
            "name": "UPGRADE_INTERFACE_VERSION",
            "outputs": [{"internalType": "string", "name": "", "type": "string"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getAdmin",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {
                    "internalType": "address",
                    "name": "_productRegistry",
                    "type": "address",
                }
            ],
            "name": "initialize",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "isAdminActive",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "owner",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "productRegistry",
            "outputs": [
                {
                    "internalType": "contract IProductRegistry",
                    "name": "",
                    "type": "address",
                }
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "proxiableUUID",
            "outputs": [{"internalType": "bytes32", "name": "", "type": "bytes32"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "renounceOwnership",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "bytes32", "name": "productId", "type": "bytes32"},
                {"internalType": "bytes", "name": "", "type": "bytes"},
            ],
            "name": "resolve",
            "outputs": [{"internalType": "int256", "name": "", "type": "int256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "bool", "name": "active", "type": "bool"}],
            "name": "setActive",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "address", "name": "admin", "type": "address"}],
            "name": "setAdmin",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "bytes32", "name": "key", "type": "bytes32"},
                {"internalType": "int256", "name": "fsp", "type": "int256"},
            ],
            "name": "submit",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "newOwner", "type": "address"}
            ],
            "name": "transferOwnership",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {
                    "internalType": "address",
                    "name": "newImplementation",
                    "type": "address",
                },
                {"internalType": "bytes", "name": "data", "type": "bytes"},
            ],
            "name": "upgradeToAndCall",
            "outputs": [],
            "stateMutability": "payable",
            "type": "function",
        },
    ],
)



import os
from datetime import datetime, timedelta
from decimal import Decimal
from pprint import pprint

import afp


AUTONITY_RPC_URL = "https://bakerloo.autonity-apis.com"
PRIVATE_KEY = os.environ["BUILDER_PRIVATE_KEY"]


def main():
    app = afp.AFP(
        rpc_url=AUTONITY_RPC_URL, authenticator=afp.PrivateKeyAuthenticator(PRIVATE_KEY)
    )
    product = app.Product()

    specification = product.create(
        symbol="SDK-TEST-1",
        description="Test Product 1",
        oracle_address="0xd8A8C5A492Fc2448cFcF980218c0F7D2De4d6FB3",
        fsv_decimals=1,
        fsp_alpha=Decimal("10000"),
        fsp_beta=Decimal("0"),
        fsv_calldata="0x",
        start_time=datetime.now() + timedelta(minutes=1),
        earliest_fsp_submission_time=datetime.now() + timedelta(days=7),
        collateral_asset="0xB855D5e83363A4494e09f0Bb3152A70d3f161940",
        tick_size=6,
        unit_value=Decimal("1"),
        initial_margin_requirement=Decimal("0.2"),
        maintenance_margin_requirement=Decimal("0.1"),
        auction_bounty=Decimal("0.1"),
        tradeout_interval=3600,
        extended_metadata="QmPK1s3pNYLi9ERiq3BDxKa4XosgWwFRQUydHUtz4YgpqB",
    )
    pprint(specification.model_dump())

    product.register(specification)
    print(product.state(specification.id))


if __name__ == "__main__":
    main()
